import 'package:flutter/material.dart';
import 'reticle_animator.dart';

class RippleAnimator extends ReticleAnimator {
  static const int DURATION_ANIMATION = 1333;
  static const int DURATION_RIPPLE_FADE_IN_MS = 333;
  static const int DURATION_RIPPLE_FADE_OUT_MS = 500;
  static const int DURATION_RIPPLE_EXPAND_MS = 833;
  static const int DURATION_RIPPLE_STROKE_WIDTH_SHRINK_MS = 833;
  static const int START_DELAY_ANIMATION = 1167;
  static const int START_DELAY_RIPPLE_FADE_OUT_MS = 667;
  static const int START_DELAY_RIPPLE_EXPAND_MS = 333;
  static const int START_DELAY_RIPPLE_STROKE_WIDTH_SHRINK_MS = 333;
  static const int TOTAL_DURATION_ANIMATION =
      DURATION_ANIMATION + START_DELAY_ANIMATION;

  Animation rippleStrokeWidthAnimation,
      rippleSizeScaleAnimation,
      rippleFadeInAnimator,
      rippleFadeOutAnimator;
  AnimationController controller;
  double rippleAlphaScale = 0.0;
  double rippleSizeScale = 0.0;
  double rippleStrokeWidthScale = 1.0;

  RippleAnimator(TickerProvider ticker) : super() {
    this.controller = AnimationController(
        duration: Duration(milliseconds: TOTAL_DURATION_ANIMATION),
        vsync: ticker);

    this.controller.addListener(() {
      notifyListeners();
    });

    // ripple fade in
    rippleFadeInAnimator = Tween(begin: 0.0, end: 1.0).animate(CurvedAnimation(
        parent: this.controller,
        curve: Interval(
            START_DELAY_ANIMATION / TOTAL_DURATION_ANIMATION,
            (START_DELAY_ANIMATION + DURATION_RIPPLE_FADE_IN_MS) /
                TOTAL_DURATION_ANIMATION,
            curve: Curves.linear)));
    rippleFadeInAnimator.addListener(() {
      if (this.controller.value <=
          (START_DELAY_ANIMATION + DURATION_RIPPLE_FADE_IN_MS) /
              TOTAL_DURATION_ANIMATION)
        this.rippleAlphaScale = rippleFadeInAnimator.value;
    });

    // ripple fade out
    rippleFadeOutAnimator = Tween(begin: 1.0, end: 0.0).animate(CurvedAnimation(
        parent: this.controller,
        curve: Interval(
            (START_DELAY_ANIMATION + START_DELAY_RIPPLE_FADE_OUT_MS) /
                TOTAL_DURATION_ANIMATION,
            1.0,
            curve: Curves.linear)));
    rippleFadeOutAnimator.addListener(() {
      if (this.controller.value >
          (START_DELAY_ANIMATION + START_DELAY_RIPPLE_FADE_OUT_MS) /
              TOTAL_DURATION_ANIMATION)
        this.rippleAlphaScale = rippleFadeOutAnimator.value;
    });

    // ripple size scale
    rippleSizeScaleAnimation = Tween(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(
            parent: controller,
            curve: Interval(
                (START_DELAY_ANIMATION + START_DELAY_RIPPLE_EXPAND_MS) /
                    TOTAL_DURATION_ANIMATION,
                (START_DELAY_ANIMATION +
                        START_DELAY_RIPPLE_EXPAND_MS +
                        DURATION_RIPPLE_EXPAND_MS) /
                    TOTAL_DURATION_ANIMATION,
                curve: Curves.fastOutSlowIn)));
    rippleSizeScaleAnimation.addListener(() {
      this.rippleSizeScale = rippleSizeScaleAnimation.value;
    });

    // ripple stroke width controller
    rippleStrokeWidthAnimation = Tween(begin: 1.0, end: 0.5).animate(
        CurvedAnimation(
            parent: this.controller,
            curve: Interval(
                (START_DELAY_ANIMATION +
                        START_DELAY_RIPPLE_STROKE_WIDTH_SHRINK_MS) /
                    TOTAL_DURATION_ANIMATION,
                (START_DELAY_ANIMATION +
                        START_DELAY_RIPPLE_STROKE_WIDTH_SHRINK_MS +
                        DURATION_RIPPLE_STROKE_WIDTH_SHRINK_MS) /
                    TOTAL_DURATION_ANIMATION,
                curve: Curves.fastOutSlowIn)));
    rippleStrokeWidthAnimation.addListener(() {
      this.rippleStrokeWidthScale = rippleStrokeWidthAnimation.value;
    });
  }

  double getRippleAlphaScale() {
    return this.rippleAlphaScale;
  }

  double getRippleSizeScale() {
    return this.rippleSizeScale;
  }

  double getRippleStrokeWidthScale() {
    return this.rippleStrokeWidthScale;
  }

  start() {
    this.controller.repeat();
  }

  stop() {
    this.controller.stop();
  }

  dispose() {
    this.controller.dispose();
    super.dispose();
  }
}
