import 'package:flutter/material.dart';
import 'reticle_state.dart';
import 'animation/reticle_animator.dart';
import 'animation/ripple_animator.dart';
import 'animation/searching_animator.dart';
import 'animation/closer_animator.dart';
import 'animation/error_animator.dart';
import 'reticle_painter.dart';

class Reticle extends StatefulWidget {
  final ReticleState state;
  final int duration;

  Reticle(this.state, {this.duration, Key key}) : super(key: key);

  @override
  _ReticleState createState() => _ReticleState();
}

class _ReticleState extends State<Reticle> with TickerProviderStateMixin {
  ReticleAnimator reticleAnimator;
  RippleAnimator rippleAnimator;
  SearchingAnimator searchingAnimator;
  CloserAnimator closerAnimator;
  ErrorAnimator errorAnimator;

  ReticleState _state;

  @override
  void initState() {
    super.initState();

    this.updateReticle();
  }

  @override
  Widget build(BuildContext context) {
    return CustomPaint(
        painter: ReticlePainter(reticleAnimator: this.reticleAnimator));
  }

  @override
  void didUpdateWidget(Reticle oldWidget) {
    if (widget.state != _state) this.updateReticle();
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    this.reticleAnimator.dispose();
    super.dispose();
  }

  void updateReticle() {
    // if we already had a reticle, we discard ressources used
    if (this.reticleAnimator != null) this.reticleAnimator.dispose();

    switch (widget.state) {
      case ReticleState.SENSING:
        // Sensing
        // shows ripple effect
        this.reticleAnimator = rippleAnimator = RippleAnimator(this)
          ..addListener(() {
            setState(() {
              this.reticleAnimator = rippleAnimator;
            });
          })
          ..start();

        break;

      case ReticleState.SEARCHING:
        // Searching
        this.reticleAnimator = searchingAnimator =
            SearchingAnimator(this, duration: widget.duration)
              ..addListener(() {
                setState(() {
                  this.reticleAnimator = searchingAnimator;
                });
              })
              ..start();

        break;

      case ReticleState.CLOSER:
        // Closer
        this.reticleAnimator = closerAnimator = CloserAnimator(this)
          ..addListener(() {
            setState(() {
              this.reticleAnimator = closerAnimator;
            });
          })
          ..start();

        break;

      case ReticleState.ERROR:
        // Error
        this.reticleAnimator = errorAnimator = ErrorAnimator(this)
          ..addListener(() {
            setState(() {
              this.reticleAnimator = errorAnimator;
            });
          })
          ..start();

        break;

      default:
    }

    _state = widget.state;
  }
}
